#
# Code by Sarah Delgado, March 2020
#
#
# InputSaver inherits ActPad_TouchVisualization:
#        GUI for Visualizing Touch Inputs (sequential and simultaneous), also shows the currently generated input-"string"
#
# InputSaver also offers the posibility to save a currently generated input string to a file
#


import tkinter
from ActPad_mpr121_controller import mpr121
from ActPad_TouchVisualization import TouchVisualization
from tkinter import messagebox


#tkinter._test()
class InputSaver(TouchVisualization):
    def __init__(self,objects,usecases):
        super().__init__(objects)
        
        self.usecase_textes = usecases
        
        self.save_frame = tkinter.Frame(self.root)
        
        #Initialize
        useCaseFrame = tkinter.Frame(self.save_frame)
        label = tkinter.Label(useCaseFrame, text = 'Select use case:')
        label.pack(side ='left')
        
        self.selectedUseCase = tkinter.IntVar()
        self.createUseCaseSelector(self.selectedUseCase,useCaseFrame,self.usecase_textes)
        useCaseFrame.pack(side ='left')
        
        save_btn = tkinter.Button(self.save_frame, text = 'Save Input', command=self.save)
        save_btn.pack(side='right',padx=10,ipadx=5, ipady=5)
        
        self.save_visible = True
        
        super().start_GUI()
        
    def createUseCaseSelector(self,radioVar,parentFrame,textes):
        radios = []
        for text in textes:
            index = textes.index(text)
            radios.append(tkinter.Radiobutton(parentFrame, text=text, 
                                              variable=radioVar, value=index+1))
            radios[len(radios)-1].pack(side = 'left',padx=10, pady=10)
    
    def start_stop(self,a):
        super().start_stop(a)
        self.toggle_save_visibility()
        
        
    def toggle_save_visibility(self):
        if self.save_visible:
            self.save_frame.pack_forget()
            self.save_visible = False
            
        else:
            self.save_frame.pack(side = 'bottom',padx=10, pady=10)
            self.start_stop_str.set('If you do not want to save, just press ENTER to start a new input.')
            self.save_visible = True
            
    def save(self):
        selected = self.selectedUseCase.get()
        if selected > 0:
            path = 'savedInput.txt'
            usecase = self.usecase_textes[selected-1]
            file_content = []
            new_file_content = ""
            with open(path, 'r') as file:
                file_content = file.readlines()
                file.close()
                
            for line in file_content:
                line = line.replace('\n','')
                if usecase in line:
                    line = usecase +','+str(self.mode.get())+','+self.input.get()
                new_file_content = new_file_content + line + '\n'
                
            with open(path, 'w') as file:    
                file.write(new_file_content)
                print(new_file_content)
                file.close()
            
            messagebox.showinfo("Input Saved", "Input saved to "+path)
            self.save_frame.pack_forget()
        
        else:
            messagebox.showerror("Error", "No use case selected!")
            

objects = {23:'Keyboard',35:'Mouse',0:'Lamp',1:'Laptop',29:'Speaker Right',30:'Plant',
                        9:'Speaker Left',19:'Monitor',20:'Stapler',10:'Android Robot',39:'Headphones'}

print('Number of connected Objects:',len(objects))

usecases = ['News','Turn Light On','Turn Monitor On','Preheat Coffe-Machine','Favorite Song','Next Song']
print('Use Cases:',usecases)

gui = InputSaver(objects,usecases = ['Update on COVID-19','Turn Light On','Turn Monitor On','Preheat Coffe-Machine','Favorite Song','Next Song'])