#!/usr/bin/env python3

#
# Code by Sarah Delgado, March 2020
#
# GUI for Visualizing Touch Inputs (sequential and simultaneous)
# This GUI also shows the currently generated input-"string"
# uncomment the last few lines to run this code independently
#

import tkinter
from ActPad_mpr121_controller import mpr121
import threading
import datetime
from tkinter import messagebox
from tkinter import font

#tkinter._test()
class TouchVisualization:
    def __init__(self,objects):       
        self.pins    = [0, 9, 10, 19, 20, 29, 30, 39,
                       1, 8, 11, 18, 21, 28, 31, 38,
                        2, 7, 12, 17, 22, 27, 32, 37,
                        3, 6, 13, 16, 23, 26, 33, 36,
                        4, 5, 14, 15, 24, 25, 34, 35]
        
        self.objects = objects
        self.input_started = False #boolean to save state of input recording
        self.simultaneously_pressed_pins = []

        #create root window
        self.master = tkinter.Tk()
        self.master.title("ActPad")
        self.root = tkinter.Frame(self.master)
        self.w, self.h = self.master.winfo_screenwidth(), self.master.winfo_screenheight()
        self.master.geometry("%dx%d" % (self.w, self.h))
        
        #Change default font
        default_font = font.nametofont("TkDefaultFont")
        default_font.configure(size=12)
        self.root.option_add("*Font", default_font)
        
        #create a variable to safe the current input
        self.input = tkinter.StringVar()
        
        #Create frames
        touchpads_frame = tkinter.Frame(self.root,bd=1, relief="sunken") #02
        objects_frame = tkinter.Frame(self.root,bd=1, relief="sunken") #03
        bottom_frame = tkinter.Frame(self.root)  #04
        
        #initiaize GUI Widgets
        self.pad_label_strings = []
        self.init_pad_widgets(touchpads_frame,self.pad_label_strings)
        
        self.object_label_strs = {}
        self.init_object_widgets(objects_frame,self.object_label_strs)
        
        self.start_stop_str = tkinter.StringVar()
        self.start_stop_str.set('Please press ENTER to start the input.')
        self.mode = tkinter.IntVar()
        self.mode.trace_add('write',self.mode_changed)
        self.init_bottom_widgets(bottom_frame,self.start_stop_str,self.mode)
                
        #Now pack the high-level-frames
        touchpads_frame.pack(padx=5, pady=15)
        objects_frame.pack(padx=5, pady=5)
        bottom_frame.pack(padx=5, pady=5, ipady=5)
        self.root.pack(padx=20, pady=20)
        
        #Add thread for picking up the touch entries
        self.quit = False
        touch_recognition_thread = threading.Thread(target=self.touch_recognition)
        touch_recognition_thread.start()
        
        #Start-GUI
        self.master.protocol("WM_DELETE_WINDOW", self.on_closing) #connect close click (X) with a function
        self.master.bind('<Return>', self.start_stop)
        
    
    def start_GUI(self):
        tkinter.mainloop()
        
    def init_pad_widgets(self,parent_frame,string_vars):
        # 02 Create touchpad widgets
        prompt_label = tkinter.Label(parent_frame, text = 'These are your Touch-Pads:')
        prompt_label.pack(padx=4,pady=4)
        
        # 02.1 Create Frames for the pads
        pad_frame_lines = [] #frame to group each line of pads
        pad_frames = []
        for i in range(len(self.pins)):
            if(i%8 == 0):
                pad_frame_lines.append(tkinter.Frame(parent_frame))
            pad_frames.append(tkinter.Frame(pad_frame_lines[int(i/8)]))
            
        
        # 02.2 Create pad widgets
        pad_labels = []
        for i in range(len(self.pins)):
            self.pad_label_strings.append(tkinter.StringVar())
            pad_labels.append(tkinter.Label(pad_frames[i], textvariable = string_vars[i], width= 3))
            #self.pad_label_strings[i].set(str(self.pins[i])+':'+)
            if self.pins[i] in self.objects:
                string_vars[i].set('O')
            else:
                string_vars[i].set('_')
               
        # 02.3 Pack pad widgets
        for i in range(len(pad_labels)):
            pad_labels[i].pack(side = 'left')
            pad_frames[i].pack(side = 'left',padx=10)

        for i in range(len(pad_frame_lines)):
            pad_frame_lines[i].pack(pady=10)
        
    def init_object_widgets(self,parent_frame,string_vars):
        titleObjs = tkinter.Label(parent_frame, text = 'Connected objects')
        titleObjs.pack(pady=5)
        
        num_obs = 0
        for obj in self.objects:
            if(num_obs%3 == 0):
                line = tkinter.Frame(parent_frame)
            str_var = tkinter.StringVar()
            str_var.set(self.objects[obj]+': _')
            string_vars[obj]=str_var
            
            frame = tkinter.Frame(line)
            label = tkinter.Label(frame, textvariable = str_var, width= 15)
            
            label.pack(side = 'left')
            frame.pack(side = 'left',padx=5)
            line.pack(pady=10)
            num_obs = num_obs +1
        
        
    def init_bottom_widgets(self,parent_frame,string_var,modeVar):
        # Create the bottom widgets
        self.createModeSelector(modeVar,parent_frame)
        
        start_stop_label=tkinter.Label(parent_frame, textvariable = string_var)
        start_stop_label.config(fg='red')
        start_stop_label.pack(side = 'top', pady=10)
        
        input_label = tkinter.Label(parent_frame, text = 'Your Input: ')
        input_label.pack(side = 'left',padx=4,pady=4)
        input_print = tkinter.Label(parent_frame, textvariable = self.input,width="50")
        input_print.pack(side = 'right',padx=5, pady=5)
        
        #self.message_string = tkinter.StringVar()
        #message_label=tkinter.Label(parent_frame, textvariable = self.message_string)
        #message_label.config(fg='red')
        #message_label.pack(side = 'bottom',padx=10, pady=10)
        
    def createModeSelector(self,radioVar,parentFrame):
        frame = tkinter.Frame(parentFrame)
        label = tkinter.Label(frame, text ='Select the desired input mode.')
        label.pack(side='left')
        textes = ['Sequential','Simultaneous']
        radios = []
        for text in textes:
            index = textes.index(text)
            radios.append(tkinter.Radiobutton(frame, text=text, 
                                              variable=radioVar, value=index+1))
            radios[len(radios)-1].pack(side = 'left',padx=5, pady=5)
        radioVar.set(1)
        frame.pack(side='top')
    
    def mode_changed(self,a,b,c):
        self.input.set('')
        if self.input_started:
            #stop input if it was started
            self.start_stop_str.set('Please press ENTER to start the input.')
            self.input_started = False
        
    def touch_recognition(self):
        mpr121_controller = mpr121(True)
        while(not self.quit):
            (touched_pin,state) = mpr121_controller.get_touch_states()
            #print(touched_pin,state)
            
            #Visualize Touches
            if(state == 0):
                    if not touched_pin in self.objects:
                        #print('Up',touched_pin)
                        self.pad_label_strings[self.pins.index(touched_pin)].set('_')
                    else:
                        #print('Up',self.objects[touched_pin])
                        self.object_label_strs[touched_pin].set(self.objects[touched_pin]+': _')
                        
                    
            if(state == 1):
                    if not touched_pin in self.objects:
                        #print('Down',touched_pin)
                        self.pad_label_strings[self.pins.index(touched_pin)].set('X')
                    else:
                        #print('Down',self.objects[touched_pin])
                        self.object_label_strs[touched_pin].set(self.objects[touched_pin]+': X')
                        
            if self.input_started:
                #print('Mode',self.mode.get())
                if self.mode.get() == 1 and state == 0:
                    #Sequential Mode, touch ended
                    self.input.set(self.input.get()+' '+str(touched_pin))
                            
                if self.mode.get() == 2:
                    #Simultaneous Mode
                    all_states = mpr121_controller.states
                    currently_pressed_pins = ""
                    i = 0
                    for state in mpr121_controller.states:
                        if state == 1:
                            currently_pressed_pins = currently_pressed_pins+' '+str(i)
                        i = i+1
                    
                    #last_input = self.input.get()
                    if len(self.input.get().split(' ')) <= len(currently_pressed_pins.split(' ')):
                        #change input only if more pins have been pressed simultaneously
                        self.input.set(currently_pressed_pins)
                        
            
        #self.quit is no true --> windows is being closed
        print('Quiting!')
        mpr121_controller.quit_loop()
        self.master.destroy()
            
    def on_closing(self):
        if messagebox.askokcancel("Quit GUI", "Do you really want to quit?!"):
            self.quit = True
    
    def start_stop(self,a):
        if(self.input_started):
            #stop recording
            self.input_started = False
            self.start_stop_str.set('Please press ENTER to start the input.')
            #self.message_string.set("Recording Stopped...")
            
        else:
            #start recording
            self.input.set('')
            self.input_started = True
            self.start_stop_str.set('Please press ENTER to stop the input.')
            #self.message_string.set("Recording Started...")
        
        
 
    
#objects = {23:'Keyboard',35:'Mouse',0:'Lamp',1:'Laptop',29:'Speaker Right',30:'Plant',
 #                       9:'Speaker Left',19:'Monitor',20:'Stapler',10:'Android Robot',39:'Headphones'}

#print('Number of connected Objects:',len(objects))

#gui = TouchVisualization(objects)
#gui.start_GUI()